@@ -1,13 +1,13 @@
 ;;; develock.el --- additional font-lock keywords for the developers
 
-;; Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007 Katsumi Yamaoka
+;; Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007, 2008 Katsumi Yamaoka
 
 ;; Author: Katsumi Yamaoka  <yamaoka@jpl.org>
 ;;         Jun'ichi Shiono  <jun@fsas.fujitsu.com>
 ;;         Yasutaka SHINDOH <ring-pub@fan.gr.jp>
 ;;         Oscar Bonilla    <ob@bitmover.com>
 ;; Created: 2001/06/28
-;; Revised: 2007/08/17
+;; Revised: 2008/04/08
 ;; Keywords: font-lock emacs-lisp change-log texinfo c java perl html
 ;;           tcl ruby mail news
 
@@ -213,7 +213,7 @@
 
 ;;; Code:
 
-(defconst develock-version "0.35"
+(defconst develock-version "0.36"
   "Version number for this version of Develock.")
 
 (require 'advice)
@@ -636,35 +636,53 @@ Each element should be triple symbols of the following form:
 \(major-mode internal-keywords user-defined-keywords)")
 
 (defvar develock-keywords-custom-type
-  (let ((args
-	 '(option :format "    %v"
-		  (list :inline t :tag "Args"
-			(radio :format "  Override: %v"
-			       (const :format "%v " nil)
-			       (const :format "%v " t)
-			       (const :format "%v " keep)
-			       (const :format "%v " prepend)
-			       (const append))
-			(boolean :tag "  LaxMatch"))))
-	(fixed-function
-	 `(list
-	   :convert-widget
-	   (lambda (widget)
-	     `(function
-	       :format "  %t: %v\n"
-	       :size 0 :value (lambda (limit))
-	       ,@(if (not (widget-get widget :copy))
-		     ;; Emacs versions prior to 21.4.
-		     '(:match
-		       (lambda (widget value) (functionp value))
-		       :value-to-internal
-		       (lambda (widget value)
-			 (widget-sexp-value-to-internal
-			  widget
-			  (if (and (stringp value)
-				   (string-match "\\`\".*\"\\'" value))
-			      (substring value 1 -1)
-			    value))))))))))
+  (let* ((args
+	  '(option :format "    %v"
+		   (list :inline t :tag "Args"
+			 (radio :format "  Override: %v"
+				(const :format "%v " nil)
+				(const :format "%v " t)
+				(const :format "%v " keep)
+				(const :format "%v " prepend)
+				(const append))
+			 (boolean :tag "  LaxMatch"))))
+	 (fixed-function
+	  `(list
+	    :convert-widget
+	    (lambda (widget)
+	      `(function
+		:format "  %t: %v\n"
+		:size 0 :value (lambda (limit))
+		,@(if (not (widget-get widget :copy))
+		      ;; Emacs versions prior to 21.4.
+		      '(:match
+			(lambda (widget value) (functionp value))
+			:value-to-internal
+			(lambda (widget value)
+			  (widget-sexp-value-to-internal
+			   widget
+			   (if (and (stringp value)
+				    (string-match "\\`\".*\"\\'" value))
+			       (substring value 1 -1)
+			     value)))))))))
+	 (face-widget
+	  (lambda (prompt)
+	    `(group
+	      :convert-widget (lambda (widget)
+				(apply 'widget-convert (widget-type widget)
+				       (eval (car (widget-get widget :args)))))
+	      (list
+	       '(const :format "" quote)
+	       (append '(face :indent 6 :format)
+		       (if (condition-case nil
+			       (string-match
+				"%{sample%}"
+				(widget-get (get 'face 'widget-type) :format))
+			     (error nil))
+			   (list (concat ,prompt "(%{sample%}) %v\n") :size 0)
+			 (list (concat ,prompt "%[select face%] %v"))))))))
+	 (face1 (funcall face-widget "  %{%t%}: "))
+	 (face2 (funcall face-widget "    %{%t%}: ")))
     `(repeat
       (choice
        :format "%[%t%] %v\n" :indent 0 :tag "Keyword Type"
@@ -672,16 +690,12 @@ Each element should be triple symbols of the following form:
 	     (const :format "  Function: %v\n"
 		    :value develock-find-long-lines)
 	     (group (const :format "" 1)
-		    (group (const :format "" quote)
-			   (face :format "\
-  %t used in a boundary (subexpression #1):\n    %[select face%] %v"
-				 :indent 6))
+		    ,(funcall face-widget "\
+  %t used in a boundary (subexpression #1):\n    ")
 		    ,args)
 	     (group (const :format "" 2)
-		    (group (const :format "" quote)
-			   (face :format "\
-  %t used out of a boundary (subexpression #2):\n    %[select face%] %v"
-				 :indent 6))
+		    ,(funcall face-widget "\
+  %t used out of a boundary (subexpression #2):\n    ")
 		    ,args))
        (list :tag "Find Tabs or Long Spaces"
 	     :sample-face widget-documentation-face
@@ -693,31 +707,24 @@ Each element should be triple symbols of the following form:
   Function: %v\n"
 		    :value develock-find-tab-or-long-space)
 	     (group (const :format "" 1)
-		    (group (const :format "" quote)
-			   (face :format "\
-  %t used for tabs before long spaces (subexpression #1):
-    %[select face%] %v"
-				 :indent 6))
+		    ,(funcall face-widget "\
+  %t used for tabs before long spaces (subexpression #1):\n    ")
 		    ,args)
 	     (group (const :format "" 2)
-		    (group (const :format "" quote)
-			   (face :format "\
-  %t used for long spaces (subexpression #2):\n    %[select face%] %v"
-				 :indent 6))
+		    ,(funcall face-widget "\
+  %t used for long spaces (subexpression #2):\n    ")
 		    ,args))
        (list :tag "(REGEXP NUM 'FACE...)"
 	     (regexp :format "  %t: %v\n" :size 0)
 	     (integer :format "  Match Subexpression Number: %v\n"
 		      :size 0)
-	     (group (const :format "" quote)
-		    (face :format "  %t: %[select face%] %v" :indent 6))
+	     ,face1
 	     ,args)
        (list :tag "(FUNCTION NUM 'FACE...)"
 	     ,fixed-function
 	     (integer :format "  Match Subexpression Number: %v\n"
 		      :size 0)
-	     (group (const :format "" quote)
-		    (face :format "  %t: %[select face%] %v" :indent 6))
+	     ,face1
 	     ,args)
        (cons :tag "(REGEXP (NUM 'FACE...) (NUM 'FACE...)...)"
 	     (regexp :format "  %t: %v\n" :size 0)
@@ -727,9 +734,7 @@ Each element should be triple symbols of the following form:
 		    (integer
 		     :format "    Match Subexpression Number: %v\n"
 		     :size 0)
-		    (group (const :format "" quote)
-			   (face :format "    %t: %[select face%] %v"
-				 :indent 6))
+		    ,face2
 		    ,args)))
        (cons :tag "(CASE_SENSITIVE_REGEXP (NUM 'FACE...)...)"
 	     (group
@@ -746,9 +751,7 @@ Each element should be triple symbols of the following form:
 		    (integer
 		     :format "    Match Subexpression Number: %v\n"
 		     :size 0)
-		    (group (const :format "" quote)
-			   (face :format "    %t: %[select face%] %v"
-				 :indent 6))
+		    ,face2
 		    ,args)))
        (cons :tag "(FUNCTION (NUM 'FACE...) (NUM 'FACE...)...)"
 	     ,fixed-function
@@ -758,9 +761,7 @@ Each element should be triple symbols of the following form:
 		    (integer
 		     :format "    Match Subexpression Number: %v\n"
 		     :size 0)
-		    (group (const :format "" quote)
-			   (face :format "    %t: %[select face%] %v"
-				 :indent 6))
+		    ,face2
 		    ,args)))
        (cons :tag "(REGEXP (NUM (IF COND 'FACE)...)...)"
 	     (regexp :format "  %t: %v\n" :size 0)
@@ -773,9 +774,7 @@ Each element should be triple symbols of the following form:
 		    (group :format "%v"
 			   (const :format "" if)
 			   (sexp :format "    Condition: %v\n" :size 0)
-			   (group (const :format "" quote)
-				  (face :format "    %t: %[select face%] %v"
-					:indent 6)))
+			   ,face2)
 		    ,args)))
        (sexp :size 0))))
   "*Customizing widget for the extraordinary level font-lock keywords.")
